# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal, TypeAlias

__all__ = ["PaymentMethodTypes"]

PaymentMethodTypes: TypeAlias = Literal[
    "ach",
    "affirm",
    "afterpay_clearpay",
    "alfamart",
    "ali_pay",
    "ali_pay_hk",
    "alma",
    "amazon_pay",
    "apple_pay",
    "atome",
    "bacs",
    "bancontact_card",
    "becs",
    "benefit",
    "bizum",
    "blik",
    "boleto",
    "bca_bank_transfer",
    "bni_va",
    "bri_va",
    "card_redirect",
    "cimb_va",
    "classic",
    "credit",
    "crypto_currency",
    "cashapp",
    "dana",
    "danamon_va",
    "debit",
    "duit_now",
    "efecty",
    "eft",
    "eps",
    "fps",
    "evoucher",
    "giropay",
    "givex",
    "google_pay",
    "go_pay",
    "gcash",
    "ideal",
    "interac",
    "indomaret",
    "klarna",
    "kakao_pay",
    "local_bank_redirect",
    "mandiri_va",
    "knet",
    "mb_way",
    "mobile_pay",
    "momo",
    "momo_atm",
    "multibanco",
    "online_banking_thailand",
    "online_banking_czech_republic",
    "online_banking_finland",
    "online_banking_fpx",
    "online_banking_poland",
    "online_banking_slovakia",
    "oxxo",
    "pago_efectivo",
    "permata_bank_transfer",
    "open_banking_uk",
    "pay_bright",
    "paypal",
    "paze",
    "pix",
    "pay_safe_card",
    "przelewy24",
    "prompt_pay",
    "pse",
    "red_compra",
    "red_pagos",
    "samsung_pay",
    "sepa",
    "sepa_bank_transfer",
    "sofort",
    "swish",
    "touch_n_go",
    "trustly",
    "twint",
    "upi_collect",
    "upi_intent",
    "vipps",
    "viet_qr",
    "venmo",
    "walley",
    "we_chat_pay",
    "seven_eleven",
    "lawson",
    "mini_stop",
    "family_mart",
    "seicomart",
    "pay_easy",
    "local_bank_transfer",
    "mifinity",
    "open_banking_pis",
    "direct_carrier_billing",
    "instant_bank_transfer",
    "billie",
    "zip",
    "revolut_pay",
    "naver_pay",
    "payco",
]
